Documentation
¶
Overview ¶
Package cli makes reusable the core parts of the squircy command.
Index ¶
- func CoreFlags(fs *flag.FlagSet, root string)
- func DefaultFlags(fs *flag.FlagSet)
- func IRCFlags(fs *flag.FlagSet)
- func PluginOptsFlag(fs *flag.FlagSet, name, usage string)
- func PluginOptsFlagVar(fs *flag.FlagSet, val flag.Value, name, usage string)
- func VMFlags(fs *flag.FlagSet)
- type Config
- type Manager
Constants ¶
This section is empty.
Variables ¶
This section is empty.
Functions ¶
func PluginOptsFlagVar ¶ added in v0.11.0
Types ¶
type Config ¶
type Config struct {
RootDir string `toml:"root_path" flag:"root"`
PluginDir string `toml:"plugin_path"`
ExtraPlugins []string `toml:"extra_plugins" flag:"plugin"`
PluginOptions map[string]interface{} `flag:"plugin_option"`
LogLevel logrus.Level `toml:"log_level"`
// Specify additional plugins that are a part of the main executable.
LinkedPlugins []plugin.Initializer
}
Source Files
¶
- flags.go
- manager.go
Click to show internal directories.
Click to hide internal directories.